use bitvec::{order::Msb0, view::BitView};

const WIDTH: i32 = 96;
const HEIGHT: i32 = 38;

/// # Safety
/// `ptr` must point to a 456-byte chunk of memory
#[no_mangle]
pub unsafe extern "C" fn next_frame(frame: u16, ptr: *mut u8) -> u8 {
    let out = unsafe { std::slice::from_raw_parts_mut(ptr, 456) };
    out.fill(0);
    let out = out.view_bits_mut::<Msb0>();

    let frame: i32 = frame.into();

    if frame > 100 {
        return 1;
    }

    let cat = include_bytes!(concat!(env!("OUT_DIR"), "/cat.bin")).view_bits::<Msb0>();
    let meow = include_bytes!(concat!(env!("OUT_DIR"), "/meow.bin")).view_bits::<Msb0>();

    let thres = WIDTH / 3;

    let dx = if frame <= 10 {
        frame * 6 - thres
    } else if frame > 87 {
        (frame - 77) * 6 - thres
    } else {
        if frame / 7 % 2 == 0 {
            // make meow flash on and off every 7 frames for the remaining time

            for x in 0..96 {
                for y in 0..38 {
                    let idx = index(x, y);

                    out.set(idx, meow[idx]);
                }
            }
        }

        60 - thres
    };

    for x in 0..96 {
        for y in 0..38 {
            let out_idx = index(x, y);
            let in_idx = index(x - dx, y + cat_height(frame));

            out.set(out_idx, cat[in_idx] || out[out_idx]);
        }
    }

    0
}

fn cat_height(frame: i32) -> i32 {
    match frame % 4 {
        0 => 0,
        1 => 1,
        2 => 2,
        3 => 1,
        _ => unreachable!(),
    }
}

fn index(x: i32, y: i32) -> usize {
    if x < 0 || y < 0 || x >= WIDTH || y >= HEIGHT {
        return 0;
    }

    x as usize + y as usize * WIDTH as usize
}
